!! Module to implement Savitzky-Golay smoothing filters.
!! Slightly modified from Numerical Recipes in Fortran, chapter 14.

module Savitzky_Golay

	use lu_decomposition

	implicit none

	contains

	subroutine savgol(c,np,nl,nr,ld,m,wrap)
	
		! Subroutine arguments
		integer,intent(in) :: np! number of filter coefficients to be computed
		integer,intent(in) :: nl! number of leftward (past) data points used
		integer,intent(in) :: nr! number of rightward (future) data points used
		integer,intent(in) :: ld! order of the derivative (0 for smoothing)
		integer,intent(in) :: m	! order of the smoothing polynomial, also 
								! equal to the highest conserved moment. 
								! Usually m=2 or 4.
		integer,intent(in) :: wrap !=1 to output coefficients in wraparound order
		real,dimension(np),intent(out) :: c		! filter coefficients
		
		! local variables
		integer		:: imj,ipj,j,k,kk,mm,indx(m+1)
		real 		:: d,fac,sum,a(m+1,m+1),b(m+1)

		!! check input parameters
		if (np<nl+nr+1.OR.nl<0.OR.nr<0.OR.ld>m.OR.m<0.OR.nl+nr<m) &
			& pause 'bad arguments in savgol'

		!! set up the normal equations of the desired least-squares fit
		do ipj=0,2*m
			sum=0.
			if (ipj==0) sum=1.	
			do k=1,nr
				sum=sum+float(k)**ipj
			end do
			do k=1,nl
				sum=sum+float(-k)**ipj
			end do
			mm=MIN(ipj,2*m-ipj)
			do imj=-mm,mm,2
				a(1+(ipj+imj)/2,1+(ipj-imj)/2)=sum
			end do
		end do

		!! solve the normal equations by LU decomposition
		call ludcmp(a,m+1,m+1,indx,d)

		!! RHS vector is unit vector depending on which derivative we want
		b=0.
		b(ld+1)=1.

		!! backsubstitute, giving one row of the inverse matrix
		call lubksb(a,m+1,m+1,indx,b)

		!! zero out the output array
		c=0.

		!! Form the Savitzky-Golay coefficients as the dot product of powers
		!! of an integer with the inverse matrix row
		do k=-nl,nr
			sum=b(1)
			fac=1.
			do mm=1,m
				fac=fac*k
				sum=sum+b(mm+1)*fac
			end do
			if (wrap) then
				kk=MOD(np-k,np)+1	!! store in wrap-around order
			else
				kk=k+nl+1
			end if
			c(kk)=sum
		end do

		return

	end subroutine savgol

end module Savitzky_Golay
