!! Module to do LU decomposition and backsubstitution.
!! Slightly modified from Numerical Recipes in Fortran, chapter 2

module lu_decomposition

	implicit none

	contains

	!---------------------------------------------------------------------
	subroutine ludcmp(a,n,np,indx,d)
	!---------------------------------------------------------------------
		
		!! subroutine arguments
		real, parameter :: TINY=1.0e-20
		integer,dimension(n),intent(out):: indx ! vector to record the
												! number of row permutations 
												! effected by the partial 
												! pivoting
		real,dimension(np,np),intent(inout):: a ! square matrix A in Ax=b
		integer,intent(in)	:: n	! dimension of block in square matrix A
		integer,intent(in)	:: np	! physical dimension of square matrix A
		real,intent(out)	:: d	! output as +/- 1 depending on
									! whether the number of row 
									! interchanges was even or odd

		!! local variables
		integer	:: i,imax,j,k
		real	:: aamax,dum,sum,vv(n)	! vv stores the implicit scaling of
										! each row

		!! loop over rows to get the implicit scaling information
		d=1.
		do i=1,n
			aamax=0.
			do j=1,n
				if(abs(a(i,j))>aamax) aamax=abs(a(i,j))
			end do
			if (aamax==0.0) pause 'singular matrix in ludcmp'
			vv(i)=1./aamax
		end do

		!! loop over columns in Crout's method
		do j=1,n
			do i=1,j-1
				sum=a(i,j)
				do k=1,i-1
					sum=sum-a(i,k)*a(k,j)
				end do
				a(i,j)=sum
			end do

			!! search for largest pivot element
			aamax=0.
			do i=j,n
				sum=a(i,j)
				do k=1,j-1
					sum=sum-a(i,k)*a(k,j)
				end do
				a(i,j)=sum
				dum=vv(i)*abs(sum) 		!! figure of merit for the pivot

				! compare with best one found so far
				if (dum>=aamax) then
					imax=i
					aamax=dum
				end if
			end do
			
			!! check for need to interchange rows
			if (j /= imax) then
				do k=1,n
					dum=a(imax,k)
					a(imax,k)=a(j,k)
					a(j,k)=dum
				end do

				!! change parity of d and interchange the scale factor
				d=-d
				vv(imax)=vv(j)
			end if
			indx(j)=imax

			!! check for singular matrix
			if (a(j,j)==0.) a(j,j)=TINY

			!! divide by the pivo element
			if (j /= n) then
				dum=1./a(j,j)
				do i=j+1,n
					a(i,j)=a(i,j)*dum
				end do
			end if
		end do

		return
	end subroutine ludcmp

	!-----------------------------------------------------------------------
	subroutine lubksb(a,n,np,indx,b)
	!-----------------------------------------------------------------------

		!! subroutine arguments
		integer,intent(in):: n		! dimension of block in square matrix A
		integer,intent(in):: np		! physical dimension of square matrix A
		integer,dimension(n),intent(in) :: indx	! permutation vector as output 
												! by subroutine ludcmp
		real,dimension(n),intent(inout) :: b	! RHS vector b in AX=b
		real,dimension(np,np),intent(in) :: a 	! matrix A as output by 
												! subroutine ludcmp (not 
												! original matrix A)	
		
		!! local variables
		integer :: i,ii,j,ll
		real :: sum

		!! When ii is set to a positive value, it will become the index of 
		!! the first nonvanishing element of b. We do forward substitution
		!! unscrumbling the permutation as we go.
		ii=0
		do i=1,n
			ll=indx(i)
			sum=b(ll)
			b(ll)=b(i)
			if (ii /=0) then
				do j=ii,i-1
					sum=sum-a(i,j)*b(j)
				end do
			else if (sum /=0.) then
				ii=i	!! a nonzero element was found so from now on we
						!! will have to do the sums in the loop above
			end if
			b(i)=sum
		end do

		!! now do the backsubstitution
		do i=n,1,-1
			sum=b(i)
			do j=i+1,n
				sum=sum-a(i,j)*b(j)
			end do
			b(i)=sum/a(i,i)
		end do
		
		return
	end subroutine lubksb
end module lu_decomposition
